@@ -2,7 +2,7 @@
 /* based on some work from Ralf Thelen and MyGnokii (www.mwiacek.com) */
 /* based on some work from MSDN */
 /* based on some work from Gnokii (www.gnokii.org)
- * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot 
+ * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot
  * GNU GPL version 2 or later
  */
 /* Due to a problem in the source code management, the names of some of
@@ -42,20 +42,20 @@
 #include "../devfunc.h"
 #include "irda.h"
 
-static bool irda_discover_device(GSM_StateMachine *state)
+static bool irda_discover_device(GSM_StateMachine *state, int *fd)
 {
 	GSM_Device_IrdaData 	*d = &state->Device.Data.Irda;
 	struct irda_device_list	*list;
     	unsigned char		*buf;
     	unsigned int		sec;
-    	int			s, z, len, fd, i;
+    	int			s, z, len, i;
     	GSM_DateTime		Date;
     	bool			founddevice = false;
 #ifdef WIN32
 	int			index;
 #endif
 
-   	fd = socket(AF_IRDA, SOCK_STREAM, 0);
+   	(*fd) = socket(AF_IRDA, SOCK_STREAM, 0);
 
     	/* can handle maximally 10 devices during discovering */
     	len  = sizeof(struct irda_device_list) + sizeof(struct irda_device_info) * 10;
@@ -70,7 +70,7 @@ static bool irda_discover_device(GSM_StateMachine *state)
 			s = len;
 			memset(buf, 0, s);
 
-			if (getsockopt(fd, SOL_IRLMP, IRLMP_ENUMDEVICES, buf, &s) == 0) {
+			if (getsockopt(*fd, SOL_IRLMP, IRLMP_ENUMDEVICES, buf, &s) == 0) {
 		    		for (i = 0; i < (int)list->numDevice; i++) {
 					dbgprintf("Irda: found device \"%s\" (address %x) - ",list->Device[i].irdaDeviceName,list->Device[i].irdaDeviceID);
 					if (strcmp(GetModelData(NULL,NULL,list->Device[i].irdaDeviceName)->number,"") != 0) {
@@ -79,7 +79,7 @@ static bool irda_discover_device(GSM_StateMachine *state)
 						if (state->CurrentConfig->Model[0]==0) strcpy(state->Phone.Data.Model,GetModelData(NULL,NULL,list->Device[i].irdaDeviceName)->number);
 						state->Phone.Data.ModelInfo = GetModelData(NULL,state->Phone.Data.Model,NULL);
 					}
-					if (founddevice) {				
+					if (founddevice) {
 			    			dbgprintf("correct\n");
 #ifdef WIN32
 			    			for(index=0; index <= 3; index++)
@@ -99,8 +99,13 @@ static bool irda_discover_device(GSM_StateMachine *state)
 		if (founddevice) break;
     	}
     	free(buf);
-    	close(fd);
-
+	if (!founddevice) {
+#ifdef WIN32
+	    	closesocket(*fd);
+#else
+	    	close(*fd);
+#endif
+	}
     	return founddevice;
 }
 
@@ -111,26 +116,23 @@ static GSM_Error irda_open (GSM_StateMachine *s)
 #ifdef WIN32
     	int 			Enable9WireMode = 1;
     	WSADATA			wsaData;
-    
+
     	WSAStartup(MAKEWORD(1,1), &wsaData);
 #else
     	if (s->ConnectionType == GCT_IRDAAT) return ERR_SOURCENOTAVAILABLE;
 #endif
 
     	/* discovering devices */
-    	if (irda_discover_device(s)==false) return ERR_TIMEOUT;
-
-    	/* Creating socket */
-    	fd = socket(AF_IRDA, SOCK_STREAM, 0);
+    	if (irda_discover_device(s,&fd)==false) return ERR_TIMEOUT;
 
     	d->peer.irdaAddressFamily 	= AF_IRDA;
 #ifndef WIN32
     	d->peer.sir_lsap_sel 		= LSAP_ANY;
 #endif
     	switch (s->ConnectionType) {
+    	case GCT_IRDAGNAPBUS:
     	case GCT_IRDAAT:
     		strcpy(d->peer.irdaServiceName, "IrDA:IrCOMM");
-
 #ifdef WIN32
 		if (setsockopt(fd, SOL_IRLMP, IRLMP_9WIRE_MODE, (const char *) &Enable9WireMode,
                		sizeof(int))==SOCKET_ERROR) return ERR_UNKNOWN;
