@@ -1,4 +1,4 @@
-/* (c) 2002-2004 by Marcin Wiacek & Michal Cihar */
+/* (c) 2002-2005 by Marcin Wiacek & Michal Cihar */
 
 #include <ctype.h>
 #include <string.h>
@@ -16,6 +16,14 @@
 #  include <signal.h>
 #endif
 
+/* Commit flag for opening files is MS extension, some other
+ * implementations (like BCC 5.5) don't like this flag at all */
+#ifdef _MSC_VER
+#  define COMMIT_FLAG "c"
+#else
+#  define COMMIT_FLAG ""
+#endif
+
 #include "gsmcomon.h"
 #include "misc/coding/coding.h"
 
@@ -83,7 +91,7 @@ unsigned char *GetMsg (INI_Section *cfg, unsigned char *default_string)
 				EncodeUnicode (buffer, buff, 5);
 			        retval = INI_GetValue(cfg, buff2, buffer, true);
 			        if (retval) {
-					sprintf(buff,"%s",DecodeUnicodeConsole(retval+2));
+					DecodeUnicode(retval+2,buff);
 					buff[strlen(buff)-1] = 0;
 					/* Set all \n to 0x0a */
 					memset(def_str,0,sizeof(def_str));
@@ -137,7 +145,7 @@ static PrintErrorEntry PrintErrorEntries[] = {
 	/* Some missed */
 	{ERR_UNKNOWNCONNECTIONTYPESTRING,"Unknown connection type string. Check config file."},
 	{ERR_UNKNOWNMODELSTRING,	"Unknown model type string. Check config file."},
-	{ERR_SOURCENOTAVAILABLE,	"Some required functions not compiled for your OS. Please contact."},
+	{ERR_SOURCENOTAVAILABLE,	"Some functions not available for your OS (disabled in config or not written)."},
 	{ERR_NOTSUPPORTED,		"Function not supported by phone."},
 	{ERR_EMPTY,			"Entry is empty"},
 	{ERR_SECURITYERROR,		"Security error. Maybe no PIN ?"},
@@ -161,8 +169,13 @@ static PrintErrorEntry PrintErrorEntries[] = {
 	{ERR_WRONGCRC,			"CRC error."},
 	{ERR_INVALIDDATETIME,		"Invalid date or time specified."},
 	{ERR_MEMORY,			"Phone memory error, maybe it is read only."},
-	{ERR_INVALIDDATA,		"Invalid data."},
+	{ERR_INVALIDDATA,		"Invalid data given to phone."},
 	{ERR_FILEALREADYEXIST,		"File with specified name already exist."},
+	{ERR_FILENOTEXIST,		"File with specified name doesn't exist."},
+	{ERR_SHOULDBEFOLDER,		"You have to give folder (not file) name."},
+	{ERR_SHOULDBEFILE,		"You have to give file (not folder) name."},
+	{ERR_NOSIM,			"Can not access SIM card."},
+	{ERR_GNAPPLETWRONG,		"Wrong GNAPPLET version in phone. Use version from currently used Gammu."},
 
 	{0,				""}
 };
@@ -209,9 +222,9 @@ GSM_Error GSM_SetDebugFile(char *info, Debug_Info *privdi)
 	if (privdi->use_global) {
 		/* Aren't we the changing the global di? */
 		if (privdi != &di) {
-			if (privdi->df != di.df && 
-					privdi->dl!=0 && 
-					fileno(privdi->df) != 1 && 
+			if (privdi->df != di.df &&
+					privdi->dl!=0 &&
+					fileno(privdi->df) != 1 &&
 					fileno(privdi->df) != 2)
 				fclose(privdi->df);
 			privdi->df = di.df;
@@ -222,14 +235,15 @@ GSM_Error GSM_SetDebugFile(char *info, Debug_Info *privdi)
         	if (privdi->df == di.df) privdi->df = stdout;
     	}
 
-	if (info[0]!=0 && privdi->dl != 0) {
+	if (info != NULL && info[0]!=0 && privdi->dl != 0) {
+		privdi->was_lf = true;
 		switch (privdi->dl) {
 		case DL_BINARY:
-			testfile = fopen(info,"wcb");
+			testfile = fopen(info,"wb" COMMIT_FLAG);
 			break;
 		case DL_TEXTERROR:
 		case DL_TEXTERRORDATE:
-			testfile = fopen(info,"ac");
+			testfile = fopen(info,"a" COMMIT_FLAG);
 			if (!testfile) {
 				dbgprintf("Can't open debug file\n");
 				return ERR_CANTOPENFILE;
@@ -237,11 +251,11 @@ GSM_Error GSM_SetDebugFile(char *info, Debug_Info *privdi)
 			fseek(testfile, 0, SEEK_END);
 			if (ftell(testfile) > 5000000) {
 				fclose(testfile);
-				testfile = fopen(info,"wc");
+				testfile = fopen(info,"w" COMMIT_FLAG);
 			}
 			break;
 		default:
-			testfile = fopen(info,"wc");
+			testfile = fopen(info,"w" COMMIT_FLAG);
 		}
 		if (!testfile) {
 			dbgprintf("Can't open debug file\n");
