@@ -1,6 +1,6 @@
-/* (c) 2001-2004 by Marcin Wiacek */
+/* (c) 2001-2006 by Marcin Wiacek */
 /* Based on some Pawel Kot and others work from Gnokii (www.gnokii.org)
- * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot 
+ * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot
  * GNU GPL version 2 or later
  */
 /* Due to a problem in the source code management, the names of some of
@@ -129,7 +129,7 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 
 		tmp=UDHHeaders[i].Length;
 		/* if length is the same */
-		if (tmp==UDH->Text[0]) { 
+		if (tmp==UDH->Text[0]) {
 
 			if (tmp==0x05) tmp=tmp-3;/*three last bytes can be different for such UDH*/
 			if (tmp==0x0b) tmp=tmp-3;/*three last bytes can be different for such UDH*/
@@ -172,7 +172,7 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 	case UDH_NokiaRingtone		   : dbgprintf("Nokia ringtone");		 		break;
 	case UDH_NokiaRingtoneLong	   : dbgprintf("Nokia ringtone");		 		break;
 	case UDH_NokiaOperatorLogo	   : dbgprintf("Nokia GSM operator logo");	 	break;
-	case UDH_NokiaCallerLogo	   : dbgprintf("Nokia caller logo");		 	break;  	
+	case UDH_NokiaCallerLogo	   : dbgprintf("Nokia caller logo");		 	break;
 	case UDH_NokiaProfileLong	   : dbgprintf("Nokia profile");		 		break;
 	case UDH_NokiaCalendarLong	   : dbgprintf("Nokia calendar note");		 	break;
 	case UDH_NokiaPhonebookLong	   : dbgprintf("Nokia phonebook entry");	 		break;
@@ -186,7 +186,7 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 		dbgprintf(", part %i of %i",UDH->PartNumber,UDH->AllParts);
 	}
 	dbgprintf("\n");
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, UDH->Text, UDH->Length);
+	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, UDH->Text, UDH->Length);
 #endif
 }
 
@@ -202,8 +202,8 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 		/* Length of UDH header */
 		off = (buffer[Layout.Text] + 1);
 		SMS->UDH.Length = off;
-		dbgprintf("UDH header available (length %i)\n",off);    
-    
+		dbgprintf("UDH header available (length %i)\n",off);
+
 		/* Copy UDH header into SMS->UDH */
 		for (i = 0; i < off; i++) SMS->UDH.Text[i] = buffer[Layout.Text + i];
 
@@ -211,11 +211,47 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 	}
 
 	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
-	if ((buffer[Layout.TPDCS] & 0xf4) == 0xf4) SMS->Coding=SMS_Coding_8bit;
-	if ((buffer[Layout.TPDCS] & 0x08) == 0x08) SMS->Coding=SMS_Coding_Unicode;
+	if ((buffer[Layout.TPDCS] & 0xC0) == 0) {
+		/* bits 7..4 set to 00xx */
+		if ((buffer[Layout.TPDCS] & 0xC) == 0xC) {
+			dbgprintf("WARNING: reserved alphabet value in TPDCS\n");
+		} else {
+			if (buffer[Layout.TPDCS] == 0) 			SMS->Coding=SMS_Coding_Default_No_Compression;
+			if ((buffer[Layout.TPDCS] & 0x2C) == 0x00) 	SMS->Coding=SMS_Coding_Default_No_Compression;
+			if ((buffer[Layout.TPDCS] & 0x2C) == 0x20) 	SMS->Coding=SMS_Coding_Default_Compression;
+			if ((buffer[Layout.TPDCS] & 0x2C) == 0x08) 	SMS->Coding=SMS_Coding_Unicode_No_Compression;
+			if ((buffer[Layout.TPDCS] & 0x2C) == 0x28) 	SMS->Coding=SMS_Coding_Unicode_Compression;
+		}
+	} else if ((buffer[Layout.TPDCS] & 0xF0) >= 0x40 &&
+		   (buffer[Layout.TPDCS] & 0xF0) <= 0xB0) {
+		/* bits 7..4 set to 0100 ... 1011 */
+		dbgprintf("WARNING: reserved coding group in TPDCS\n");
+	} else if (((buffer[Layout.TPDCS] & 0xF0) == 0xC0) ||
+	      	   ((buffer[Layout.TPDCS] & 0xF0) == 0xD0)) {
+		/* bits 7..4 set to 1100 or 1101 */
+		if ((buffer[Layout.TPDCS] & 4) == 4) {
+			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
+		} else {
+			SMS->Coding=SMS_Coding_Default_No_Compression;
+		}
+	} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xE0) {
+		/* bits 7..4 set to 1110 */
+		if ((buffer[Layout.TPDCS] & 4) == 4) {
+			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
+		} else {
+			SMS->Coding=SMS_Coding_Unicode_No_Compression;
+		}
+	} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xF0) {
+		/* bits 7..4 set to 1111 */
+		if ((buffer[Layout.TPDCS] & 8) == 8) {
+			dbgprintf("WARNING: set reserved bit 3 in TPDCS\n");
+		} else {
+			if ((buffer[Layout.TPDCS] & 4) == 0) SMS->Coding=SMS_Coding_Default_No_Compression;
+		}
+	}
 
 	switch (SMS->Coding) {
-		case SMS_Coding_Default:
+		case SMS_Coding_Default_No_Compression:
 			i = 0;
 			do {
 				i+=7;
@@ -232,18 +268,21 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 			memcpy(SMS->Text,buffer+(Layout.Text+off),SMS->Length);
 #ifdef DEBUG
 			dbgprintf("8 bit SMS, length %i\n",SMS->Length);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, SMS->Text, SMS->Length);
+			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, SMS->Text, SMS->Length);
 #endif
 			break;
-		case SMS_Coding_Unicode:
+		case SMS_Coding_Unicode_No_Compression:
 			SMS->Length=(buffer[Layout.TPUDL] - off) / 2;
 			DecodeUnicodeSpecialNOKIAChars(SMS->Text,buffer+(Layout.Text+off), SMS->Length);
 #ifdef DEBUG
 			dbgprintf("Unicode SMS, length %i\n",SMS->Length);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, buffer+(Layout.Text+off), SMS->Length*2);
+			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, buffer+(Layout.Text+off), SMS->Length*2);
 			dbgprintf("%s\n",DecodeUnicodeString(SMS->Text));
 #endif
 			break;
+		default:
+			SMS->Length=0;
+			break;
 	}
 
 	return ERR_NONE;
@@ -251,11 +290,12 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 
 GSM_Error GSM_DecodeSMSFrameStatusReportData(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
-	SMS->DeliveryStatus = buffer[Layout.TPStatus];
-    
+	SMS->DeliveryStatus 	= buffer[Layout.TPStatus];
+	SMS->Coding 		= SMS_Coding_Unicode_No_Compression;
+
 	if (buffer[Layout.TPStatus] < 0x03) {
 		EncodeUnicode(SMS->Text,"Delivered",9);
-		SMS->Length = 9;      
+		SMS->Length = 9;
 	} else if (buffer[Layout.TPStatus] & 0x40) {
 		EncodeUnicode(SMS->Text,"Failed",6);
 		SMS->Length = 6;
@@ -308,7 +348,7 @@ GSM_Error GSM_DecodeSMSFrameStatusReportData(GSM_SMSMessage *SMS, unsigned char
         case 0x64: dbgprintf("Quality of service not available");			break;
         case 0x65: dbgprintf("Error in SME");						break;
         default  : dbgprintf("Reserved/Specific to SC: %x",buffer[Layout.TPStatus]);	break;
-	}          
+	}
 	dbgprintf("\n");
 #endif
 
@@ -332,7 +372,7 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 #endif
 
 	SMS->UDH.Type 			= UDH_NoUDH;
-	SMS->Coding 			= SMS_Coding_Default;
+	SMS->Coding 			= SMS_Coding_8bit;
 	SMS->Length			= 0;
 	SMS->SMSC.Location		= 0;
 	SMS->SMSC.DefaultNumber[0]	= 0;
@@ -378,10 +418,22 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	}
 	SMS->Class = -1;
 	if (Layout.TPDCS != 255) {
-		if ((buffer[Layout.TPDCS] & 0xF3)==0xF0) SMS->Class = 0;
-		if ((buffer[Layout.TPDCS] & 0xF3)==0xF1) SMS->Class = 1;
-		if ((buffer[Layout.TPDCS] & 0xF3)==0xF2) SMS->Class = 2;
-		if ((buffer[Layout.TPDCS] & 0xF3)==0xF3) SMS->Class = 3;
+		/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
+		if ((buffer[Layout.TPDCS] & 0xD0) == 0x10) {
+			/* bits 7..4 set to 00x1 */
+			if ((buffer[Layout.TPDCS] & 0xC) == 0xC) {
+				dbgprintf("WARNING: reserved alphabet value in TPDCS\n");
+			} else {
+				SMS->Class = (buffer[Layout.TPDCS] & 3);
+			}
+		} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xF0) {
+			/* bits 7..4 set to 1111 */
+			if ((buffer[Layout.TPDCS] & 8) == 8) {
+				dbgprintf("WARNING: set reserved bit 3 in TPDCS\n");
+			} else {
+				SMS->Class = (buffer[Layout.TPDCS] & 3);
+			}
+		}
 	}
 	dbgprintf("SMS class: %i\n",SMS->Class);
 
@@ -441,7 +493,7 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 		memcpy(buffer+Layout.Text, SMS->UDH.Text, off);		/* we copy the udh */
 #ifdef DEBUG
 		dbgprintf("UDH, length %i\n",off);
-		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, SMS->UDH.Text, off);
+		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, SMS->UDH.Text, off);
 #endif
 	}
 	switch (SMS->Coding) {
@@ -449,15 +501,15 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 			/* the mask for the 8-bit data */
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
-			buffer[Layout.TPDCS] |= 0xf4;    
+			buffer[Layout.TPDCS] |= 0xf4;
 			memcpy(buffer+(Layout.Text+off), SMS->Text, SMS->Length);
 			size2 = size = SMS->Length+off;
 #ifdef DEBUG
 			dbgprintf("8 bit SMS, length %i\n",SMS->Length);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, SMS->Text, SMS->Length);
+			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, SMS->Text, SMS->Length);
 #endif
 			break;
-		case SMS_Coding_Default:
+		case SMS_Coding_Default_No_Compression:
 			p = 0;
 			do {
 				p+=7;
@@ -474,7 +526,7 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 				size = 0; size2 = 0;
 			}
 			break;
-		case SMS_Coding_Unicode:
+		case SMS_Coding_Unicode_No_Compression:
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
 			buffer[Layout.TPDCS] |= 0x08;
@@ -482,10 +534,12 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 			size=size2=UnicodeLength(buffer+(Layout.Text+off))*2+off;
 #ifdef DEBUG
 			dbgprintf("Unicode SMS, length %i\n",(size2-off)/2);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, buffer+(Layout.Text+off), size2-off);
+			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, buffer+(Layout.Text+off), size2-off);
 			dbgprintf("%s\n",DecodeUnicodeString(buffer+(Layout.Text+off)));
 #endif
 			break;
+		default:
+			break;
 	}
 
 	/* GSM 03.40 section 9.2.3.16 (TP-User-Data-Length)
@@ -536,7 +590,7 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 		buffer[Layout.SMSCNumber]=GSM_PackSemiOctetNumber(SMS->SMSC.Number,buffer+(Layout.SMSCNumber+1), false);
 		dbgprintf("SMSC number \"%s\"\n",DecodeUnicodeString(SMS->SMSC.Number));
 	}
-  
+
 	/* Message Class*/
 	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
 	if (Layout.TPDCS != 255) {
@@ -576,7 +630,7 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	/* size is the length of the data in octets including UDH */
 	*length=GSM_EncodeSMSFrameText(SMS,buffer,Layout);
 //	if (*length == 0) return GE_UNKNOWN;
-	*length += Layout.Text;	
+	*length += Layout.Text;
 
 	return ERR_NONE;
 }
@@ -587,6 +641,12 @@ void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
 {
 	SMS->Class			= -1;
 	SMS->SMSC.Location		= 1;
+	SMS->SMSC.Name[0]		= 0;
+	SMS->SMSC.Name[1]		= 0;
+	SMS->SMSC.Number[0]		= 0;
+	SMS->SMSC.Number[1]		= 0;
+	SMS->SMSC.DefaultNumber[0]	= 0;
+	SMS->SMSC.DefaultNumber[1]	= 0;
 	SMS->SMSC.Format		= SMS_FORMAT_Text;
 	SMS->SMSC.Validity.Format	= SMS_Validity_RelativeFormat;
 	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
@@ -598,7 +658,7 @@ void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
 	SMS->UDH.ID16bit		= 0;
 	SMS->UDH.PartNumber		= 0;
 	SMS->UDH.AllParts		= 0;
-	SMS->Coding			= SMS_Coding_Default;
+	SMS->Coding			= SMS_Coding_8bit;
 	SMS->Text[0] 			= 0;
 	SMS->Text[1] 			= 0;
 	SMS->PDU			= SMS_Submit;
@@ -606,12 +666,21 @@ void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
 	SMS->MessageReference		= 0;
 	SMS->ReplaceMessage		= 0;
 	SMS->Length			= 0;
+	SMS->DeliveryStatus		= 0;
+	SMS->ReplyViaSameSMSC		= 0;
+	SMS->Class			= 0;
+	SMS->MessageReference		= 0;
+	SMS->ReplaceMessage		= 0;
+	SMS->RejectDuplicates		= 0;
 
-	/* This part is required to save SMS */    
+	/* This part is required to save SMS */
 	SMS->State			= SMS_UnSent;
 	SMS->Location			= 0;
+	SMS->Memory			= 0;
 	SMS->Folder			= 0x02;	/*Outbox*/
+	SMS->InboxFolder		= false;
 	GSM_GetCurrentDateTime (&SMS->DateTime);
+	GSM_GetCurrentDateTime (&SMS->SMSCTime);
 	SMS->Name[0]			= 0;
 	SMS->Name[1]			= 0;
 }
@@ -670,6 +739,46 @@ void GSM_EncodeUDHHeader(GSM_UDHHeader *UDH)
 	}
 }
 
+bool GSM_DecodeSiemensOTASMS(GSM_SiemensOTASMSInfo	*Info,
+			     GSM_SMSMessage 		*SMS)
+{
+	int current;
+
+	if (SMS->PDU != SMS_Deliver) 		return false;
+	if (SMS->Coding != SMS_Coding_8bit) 	return false;
+	if (SMS->Class != 1) 			return false;
+	if (SMS->UDH.Type != UDH_NoUDH) 	return false;
+	if (SMS->Length < 22) 			return false;
+
+	if (strncmp(SMS->Text,"//SEO",5)!=0) return false; //Siemens Exchange Object
+	if (SMS->Text[5]!=1) return false; //version 1
+	Info->DataLen = SMS->Text[6] + SMS->Text[7]*256;
+	Info->SequenceID = SMS->Text[8] + SMS->Text[9]*256 +
+			 SMS->Text[10]*256*256 + SMS->Text[11]*256*256*256;
+	Info->PacketNum = SMS->Text[12] + SMS->Text[13]*256;
+	Info->PacketsNum = SMS->Text[14] + SMS->Text[15]*256;
+	dbgprintf("Packet %i/%i\n",Info->PacketNum,Info->PacketsNum);
+	Info->AllDataLen = SMS->Text[16] + SMS->Text[17]*256 +
+			 SMS->Text[18]*256*256 + SMS->Text[19]*256*256*256;
+	dbgprintf("DataLen %i/%lu\n",Info->DataLen,Info->AllDataLen);
+
+	if (SMS->Text[20] > 9) return false;
+	memcpy(Info->DataType,SMS->Text+21,SMS->Text[20]);
+	Info->DataType[SMS->Text[20]] = 0;
+	dbgprintf("DataType '%s'\n",Info->DataType);	
+
+	current = 21+SMS->Text[20];
+	if (SMS->Text[current] > 39) return false;
+	memcpy(Info->DataName,SMS->Text+current+1,SMS->Text[current]);
+	Info->DataName[SMS->Text[current]] = 0;
+	dbgprintf("DataName '%s'\n",Info->DataName);
+
+	current += SMS->Text[current]+1;
+	memcpy(Info->Data,SMS->Text+current,Info->DataLen);
+
+	return true;
+}
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
