@@ -10,7 +10,7 @@
 #include "../../gammu.h"
 #include "../../../common/phone/pfunc.h"
 #include "../../../common/phone/nokia/nfunc.h"
-#include "../../../common/phone/nokia/dct4/dct4func.h"
+#include "../../../common/phone/nokia/dct4tiku/dct4func.h"
 #include "../../../common/misc/coding/coding.h"
 
 extern GSM_Reply_Function UserReplyFunctions4[];
@@ -36,7 +36,7 @@ GSM_Error CheckDCT4Only()
 	if (s.ConnectionType!=GCT_MBUS2	     && s.ConnectionType!=GCT_FBUS2      &&
 	    s.ConnectionType!=GCT_FBUS2DLR3  && s.ConnectionType!=GCT_PHONETBLUE &&
 	    s.ConnectionType!=GCT_IRDAPHONET && s.ConnectionType!=GCT_BLUEPHONET &&
-	    s.ConnectionType!=GCT_FBUS2DKU5) {
+	    s.ConnectionType!=GCT_DKU5FBUS2  && s.ConnectionType!=GCT_DKU2PHONET) {
 		return ERR_OTHERCONNECTIONREQUIRED;
 	}
 	return ERR_NONE;
@@ -106,6 +106,7 @@ static DCT4_Feature DCT4Features[] = {
 	{DCT4_JAVA_GAMES_MENU,		 "Java games menu in Games",	 {{1,"on"},{0,"off"},{0,""}}},
 	{DCT4_SAT_CONFIRM_MENU,		 "Can use confirming SIM service actions", {{1,"on"},{0,"off"},{0,""}}},
 	{DCT4_INSTANT_MESS_MENU,	 "Instant Messaging in Messages",{{1,"on"},{0,"off"},{0,""}}},
+	{DCT4_INSTANT_MESS_MENU2,	 "Instant Messaging in Messages",{{0,"on"},{1,"off"},{0,""}}},
 	{DCT4_CONFIRM_ALS,		 "Confirm using ALS",            {{1,"on"},{0,"off"},{0,""}}},
 	{DCT4_BOOKMARK_GOTO_MENU,	 "Bookmarks in GoTo menu",       {{1,"on"},{0,"off"},{0,""}}},
 
@@ -184,6 +185,18 @@ static DCT4_Phone_Features DCT4PhoneFeatures[] = {
 			 {DCT4_WAP_BOOKMARKS_MENU2,24},{DCT4_SERVICES_GAMES_APP_GALLERY,25},
 			 {DCT4_3GINDICATOR,27},{DCT4_DISPLAY_WAP_PROFILE,30},{DCT4_SAT_CONFIRM_MENU,32},
 			 {DCT4_CONFIRM_ALS,33},{DCT4_JAVA_TCK,36},{DCT4_BOOKMARK_GOTO_MENU,37},
+			 {DCT4_INSTANT_MESS_MENU2,38},
+			 {0,0}}},
+/*6230*/ {"RH-12",	{{DCT4_ALS,1},{DCT4_A52,3},{DCT4_CSP,4},{DCT4_GEA1,9},{DCT4_DISPLAY_PHONE_NAME,17},
+			 {DCT4_WAP_GOTO_MENU,18},{DCT4_WAP_SETTINGS_MENU,19},
+			 {DCT4_SERVICES_GAMES_APP_GALLERY,21},{DCT4_DISPLAY_WAP_PROFILE,25},
+			 {DCT4_SAT_CONFIRM_MENU,26},{DCT4_CONFIRM_ALS,39},{DCT4_WAP_BOOKMARKS_MENU2,40},
+			 {DCT4_BLUETOOTH_MENU,35},{DCT4_JAVA_TCK,57},{DCT4_BOOKMARK_GOTO_MENU,61},
+			 //{DCT4_TEST2,13},//number grouping
+			 //{DCT4_5100_IDENTIFY,30},//number grouping
+			 //{DCT4_TEST,48},//Chat menu in main menu
+			 {0,0}}},
+/*6230i*/ {"RM-72",	{{DCT4_ALS,1},{DCT4_CONFIRM_ALS,39},
 			 {0,0}}},
 /*6310*/ {"NPE-4",	{{DCT4_ALS,1},{DCT4_A52,3},{DCT4_CSP,6},{DCT4_GAMES_WAP_DOWNLOAD,7},
 			 {DCT4_GAMES_SCORE_SEND,8},{DCT4_GAMES_URL_CHECK,9},{DCT4_BLUETOOTH_MENU,10},
@@ -347,7 +360,7 @@ static GSM_Error DCT4_ReplyTestsStatus(GSM_Protocol_Message msg, GSM_StateMachin
 					default  : printf("Unknown (%x)",msg.Buffer[pos+3]);
 				}
 				if (DCT4Tests.Tests[j].Startup) printf(" (startup)");
-				printf("\n");				
+				printf("\n");
 				break;
 			}
 		}
@@ -363,7 +376,7 @@ void DCT4SelfTests(int argc, char *argv[])
 	unsigned char GetDoneST[6]    = {0x00, 0x08, 0x01, 0x04, 0x01, 0x00};
 	unsigned char GetDoneST2[6]   = {0x00, 0x08, 0x02, 0x04, 0x02, 0x00};
 	unsigned char GetNames[6]     = {0x00, 0x08, 0x03, 0x06, 0x03, 0x00};
-	unsigned char GetStatus[6]    = {0x00, 0x08, 0x04, 0x02, 0x03, 0x00}; 
+	unsigned char GetStatus[6]    = {0x00, 0x08, 0x04, 0x02, 0x03, 0x00};
 
 	unsigned char RunALL[6]       = {0x00, 0x06, 0x04, 0x00, 0x03, 0x00};
 
@@ -531,10 +544,10 @@ static GSM_Error DCT4_ReplyGetSecurityCode(GSM_Protocol_Message msg, GSM_StateMa
 void DCT4GetSecurityCode(int argc, char *argv[])
 {
 	GSM_Error 	error;
-	unsigned char 	getlen[]={0x00, 0x08, 0x01, 0x0C, 
-				  0x00, 0x23, 		//ID 
+	unsigned char 	getlen[]={0x00, 0x08, 0x01, 0x0C,
+				  0x00, 0x23, 		//ID
 				  0x00, 0x00, 		//Index
-				  0x00, 0x00}; 
+				  0x00, 0x00};
 	unsigned char 	read[]={0x00, 0x08, 0x02, 0x04,
 				0x00, 0x23, 		//ID
 				0x00, 0x00, 		//Index
@@ -669,7 +682,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 	dbgprintf("Getting voice record name\n");
 	error=GSM_WaitFor (&s, ReqNames, 14, 0x4A, 4, ID_User4);
 	Print_Error(error);
-	
+
 	s.Phone.Data.PhoneString 	= Buffer;
 	ReqToken[7] 			= Location;
 	dbgprintf("Getting voice record token\n");
@@ -678,7 +691,7 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 	TokenLocation 			= Buffer[0] * 256 + Buffer[1];
 	Token				= Buffer[2];
 
-	WAVFile = fopen(FileName, "wb");      
+	WAVFile = fopen(FileName, "wb");
 
 	fwrite(&WAV_Header,	1, sizeof(WAV_Header),	WAVFile);
 	fwrite(&FMT_Header,	1, sizeof(FMT_Header),	WAVFile);
@@ -770,7 +783,8 @@ static GSM_Error DCT4_ReplyGetBTInfo(GSM_Protocol_Message msg, GSM_StateMachine
 
 static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	int i;
+	int 			i;
+	unsigned char 		buff[7];
 
 	switch (msg.Buffer[3]) {
 	case 0x0D:
@@ -779,12 +793,21 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 		for (i=14;i<22;i++) {
 			dbgprintf("%02x",msg.Buffer[i]);
 		}
+		sprintf(buff,"%02x%02x%02x",msg.Buffer[14],msg.Buffer[15],msg.Buffer[16]);
+		buff[6] = 0;
+		buff[5] = buff[4];
+		buff[4] = buff[3];
+		buff[3] = ' ';
+		if (strcmp(DecodeUnicodeString(GSM_GetNetworkName(buff)),"unknown")) {
+			printf("Old simlock   : %s (%s)\n",DecodeUnicodeString(GSM_GetNetworkName(buff)),buff);
+		}
+
 		dbgprintf("\n");
 		dbgprintf("Profile_Bits: ");
 		for (i=22;i<30;i++) {
 			dbgprintf("%02x",msg.Buffer[i]);
 		}
-		dbgprintf("\n");		
+		dbgprintf("\n");
 		return ERR_NONE;
 	case 0x13:
 		dbgprintf("Simlock info received\n");
@@ -858,7 +881,7 @@ void DCT4GetT9(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	T9File = fopen("T9", "w");      
+	T9File = fopen("T9", "w");
 	if (T9File == NULL) return;
 
 	s.User.UserReplyFunctions=UserReplyFunctions4;
@@ -874,7 +897,7 @@ void DCT4GetT9(int argc, char *argv[])
 			}
 			if (T9Dictionary - i == 0) break;
 		}
-		error=GSM_WaitFor (&s, req, 18, 0x23, 4, ID_User3);		
+		error=GSM_WaitFor (&s, req, 18, 0x23, 4, ID_User3);
 		Print_Error(error);
 		if (i==0) {
 			T9Dictionary = T9FullSize;
@@ -978,7 +1001,7 @@ struct DCT4ADCInfo {
 static struct DCT4ADCInfo DCT4ADC[] = {
 	{"Battery voltage, divided:",		"mV",  1},
 	{"Battery voltage, scaled:",		"mV",  1},
-	{"Charger voltage:",			"mV",  1},  	
+	{"Charger voltage:",			"mV",  1},
 	{"Charger current:",			"mA",  1},
 	{"Battery size indicator:",		"Ohms",100},
 	{"Battery temperature:",		"K",   1},
@@ -1013,11 +1036,11 @@ void DCT4GetADC(int argc, char *argv[])
 	while (1) {
 		printf(" %30s ",DCT4ADC[i].name);
 		GetRaw[4] = i;
-		error=GSM_WaitFor (&s, GetRaw, 6, 0x17, 4, ID_User3);		
+		error=GSM_WaitFor (&s, GetRaw, 6, 0x17, 4, ID_User3);
 		Print_Error(error);
 		GetUnit[4] 	= i;
 		ADC		= DCT4ADC[i].x;
-		error=GSM_WaitFor (&s, GetUnit, 6, 0x17, 4, ID_User3);		
+		error=GSM_WaitFor (&s, GetUnit, 6, 0x17, 4, ID_User3);
 		Print_Error(error);
 		printf("%s\n",DCT4ADC[i].unit);
 		i++;
@@ -1097,7 +1120,7 @@ void DCT4TuneRadio(int argc, char *argv[])
 	if (error != ERR_NONE && error != ERR_EMPTY) {
 		printf("Phone seems not to support radio\n");
 		GSM_Terminate();
-		exit(-1);		
+		exit(-1);
 	}
 
 	error=GSM_WaitFor (&s, Enable, 6, 0x3E, 4, ID_User3);
@@ -1184,7 +1207,7 @@ void DCT4TuneRadio(int argc, char *argv[])
 
 void DCT4PlaySavedRingtone(int argc, char *argv[])
 {
-	unsigned char req[] =  {N6110_FRAME_HEADER, 
+	unsigned char req[] =  {N6110_FRAME_HEADER,
 			      	0x01,
 				0x00,0x64,	//id
 				0x01,		//group
@@ -1192,6 +1215,7 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 				0x0A,		//volume
 				0x00,0x00,0x00,0x00,0x00,0x00,0x00};
 	GSM_AllRingtonesInfo Info;
+//	int i;
 
 	GSM_Init(true);
 
@@ -1206,9 +1230,9 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 		GSM_Terminate();
 		return;
 	}
-	req[4] = Info.Ringtone[atoi(argv[2])].ID / 256;
-	req[5] = Info.Ringtone[atoi(argv[2])].ID % 256;
-	req[6] = Info.Ringtone[atoi(argv[2])].Group;
+	req[4] = Info.Ringtone[atoi(argv[2])-1].ID / 256;
+	req[5] = Info.Ringtone[atoi(argv[2])-1].ID % 256;
+	req[6] = Info.Ringtone[atoi(argv[2])-1].Group;
 
 	error=GSM_WaitFor (&s, req, 18, 0x1F, 4, ID_User3);
 	Print_Error(error);
@@ -1220,7 +1244,7 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 
 static GSM_Error DCT4_ReplyMakeCameraShoot(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	return ERR_NONE;	
+	return ERR_NONE;
 }
 
 void DCT4MakeCameraShoot(int argc, char *argv[])
@@ -1229,9 +1253,9 @@ void DCT4MakeCameraShoot(int argc, char *argv[])
 	unsigned char CameraON[] = {N6110_FRAME_HEADER, 0x02, 0x01, 0x00, 0x00, 0x00 , 0x00, 0x00};
 	unsigned char CameraON2[] = {N6110_FRAME_HEADER, 0xF0, 0x02, 0x00};
 	unsigned char MakeShot[200] = {N6110_FRAME_HEADER, 0x06, 0x01, 0x06,
-		0x01, 0x00, 0x00, 0x02, 0x00, 0x04, 0x32, 0x00, 0x01, 
+		0x01, 0x00, 0x00, 0x02, 0x00, 0x04, 0x32, 0x00, 0x01,
 		0x1D, 		//length of rest
-		0x00, 0x00, 0x00, 0x01, 
+		0x00, 0x00, 0x00, 0x01,
 		0x00, 0x02,	//master folder id
 		0x00, 0x14}; 	//length
 	unsigned char CameraOFF[] = {N6110_FRAME_HEADER, 0x04, 0x01, 0x00};
@@ -1266,7 +1290,7 @@ int len;
 static GSM_Error DCT4_ReplyGetScreenDump(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
 	if (msg.Buffer[7] == 0x0C) len = 1;
-	return ERR_NONE;	
+	return ERR_NONE;
 }
 
 void DCT4GetScreenDump(int argc, char *argv[])
@@ -1288,7 +1312,106 @@ void DCT4GetScreenDump(int argc, char *argv[])
 	GSM_Terminate();
 }
 
-static GSM_Reply_Function UserReplyFunctions4[] = {
+static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMachine *s)
+{
+	int i,pos=6;
+
+	printf("%i entries types\n",msg.Buffer[5]-1);
+
+	for (i=0;i<msg.Buffer[5]-1;i++) {
+		printf("  entry ID %02X",msg.Buffer[pos+4]);
+		switch (msg.Buffer[pos+4]) {
+		case N7110_PBK_SIM_SPEEDDIAL	: printf(" (Speed dial on SIM)"); 			break;
+		case N7110_PBK_NAME	    	: printf(" (Text: name (always the only one)");		break;
+		case N7110_PBK_EMAIL	    	: printf(" (Text: email adress)");			break;
+		case N7110_PBK_POSTAL	    	: printf(" (Text: postal address)");			break;
+		case N7110_PBK_NOTE	    	: printf(" (Text: note)");				break;
+		case N7110_PBK_NUMBER 	    	: printf(" (Phone number)");				break;
+		case N7110_PBK_RINGTONE_ID  	: printf(" (Ringtone ID)");				break;
+		case N7110_PBK_DATETIME    	: printf(" (Call register: date and time)");		break;
+		case N7110_PBK_UNKNOWN1	    	: printf(" (Call register: with missed calls)");	break;
+		case N7110_PBK_SPEEDDIAL    	: printf(" (Speed dial)");				break;
+		case N7110_PBK_GROUPLOGO    	: printf(" (Caller group: logo)");			break;
+		case N7110_PBK_LOGOON	    	: printf(" (Caller group: is logo on ?)");		break;
+		case N7110_PBK_GROUP	    	: printf(" (Caller group number in pbk entry)");	break;
+	
+		/* DCT4 only */
+		case N6510_PBK_URL		: printf(" (Text: URL address)");			break;
+		case N6510_PBK_SMSLIST_ID	: printf(" (SMS list assigment)");			break;
+		case N6510_PBK_VOICETAG_ID	: printf(" (Voice tag assigment)");			break;
+		case N6510_PBK_PICTURE_ID	: printf(" (Picture ID assigment)");			break;
+		case N6510_PBK_RINGTONEFILE_ID  : printf(" (Ringtone ID from filesystem/internal)");	break;
+		case N6510_PBK_USER_ID          : printf(" (Text: user ID)");				break;
+		case N6510_PBK_UNKNOWN2	        : printf(" (conversation list ID)");			break;
+		case N6510_PBK_UNKNOWN3	 	: printf(" (Instant Messaging service list ID ?)");	break;
+		case N6510_PBK_UNKNOWN4	 	: printf(" (presence list ID ?)");			break;
+		case N6510_PBK_PUSHTOTALK_ID	: printf(" (SIP Address (Push to Talk address))");	break;
+		case N6510_PBK_GROUP2_ID	: printf(" (Group ID (6230i or later))");		break;
+		}
+		printf(", type ");
+		switch (msg.Buffer[pos+5]) {
+		case 0x05: printf("string"); 	break;
+		case 0x02: printf("byte"); 	break;
+		case 0x03: printf("2 bytes"); 	break;
+		case 0x06: printf("4 bytes"); 	break;
+		default  : printf("%02X",msg.Buffer[pos+5]);
+		}
+		printf("\n");
+		pos+=msg.Buffer[pos+3];
+	}
+
+	printf("%i phone number types\n",msg.Buffer[pos+4]);
+	for (i=0;i<msg.Buffer[pos+4];i++) {
+		switch (msg.Buffer[pos+5+i]) {
+		case 0x02: printf("  Home Number\n"); 		break;
+		case 0x03: printf("  Mobile Number\n"); 	break;
+		case 0x04: printf("  Fax Number\n"); 		break;
+		case 0x06: printf("  Office Number\n"); 	break;
+		case 0x0A: printf("  Standard Number\n"); 	break;
+		default:   printf("  unknown\n");
+		}
+	}
+
+	return ERR_NONE;
+}
+
+void DCT4GetPBKFeatures(int argc, char *argv[])
+{
+	GSM_MemoryType	MemoryType=0;
+	unsigned char 	req[] = {N6110_FRAME_HEADER, 0x25, 
+				 0x05, 	// memory type
+				 0x00};
+
+	if (mystrncasecmp(argv[2],"DC",0)) MemoryType=MEM_DC;
+	if (mystrncasecmp(argv[2],"ON",0)) MemoryType=MEM_ON;
+	if (mystrncasecmp(argv[2],"RC",0)) MemoryType=MEM_RC;
+	if (mystrncasecmp(argv[2],"MC",0)) MemoryType=MEM_MC;
+	if (mystrncasecmp(argv[2],"ME",0)) MemoryType=MEM_ME;
+	if (mystrncasecmp(argv[2],"SM",0)) MemoryType=MEM_SM;
+	if (mystrncasecmp(argv[2],"VM",0)) MemoryType=MEM_VM;
+	if (mystrncasecmp(argv[2],"FD",0)) MemoryType=MEM_FD;
+	if (MemoryType==0) {
+		printf("ERROR: unknown memory type (\"%s\")\n",argv[2]);
+		exit (-1);
+	}
+
+	req[4] = NOKIA_GetMemoryType(&s, MemoryType,N71_65_MEMORY_TYPES);
+	if (req[4]==0xff) exit(-1);
+
+	GSM_Init(true);
+
+        CheckDCT4();
+
+	s.User.UserReplyFunctions=UserReplyFunctions4;
+
+	error=GSM_WaitFor (&s, req, 6, 0x03, 4, ID_User3);
+	Print_Error(error);
+
+	GSM_Terminate();
+}
+
+GSM_Reply_Function UserReplyFunctions4[] = {
+	{DCT4_ReplyGetPBKFeatures,	"\x03",0x03,0x26,ID_User3	},	
 
 #ifdef DEBUG
 	{DCT4_ReplyResetSecurityCode,	"\x08",0x03,0x05,ID_User2	},
@@ -1313,7 +1436,7 @@ static GSM_Reply_Function UserReplyFunctions4[] = {
 	{DCT4_ReplyGetT9,		"\x23",0x03,0x05,ID_User3	},
 	{DCT4_ReplyGetVoiceRecord,	"\x23",0x03,0x05,ID_User4	},
 	{DCT4_ReplyGetVoiceRecord,	"\x23",0x03,0x0D,ID_User4	},
-	{DCT4_ReplyGetSecurityCode,	"\x23",0x03,0x0D,ID_User1	}, 
+	{DCT4_ReplyGetSecurityCode,	"\x23",0x03,0x0D,ID_User1	},
 
 	{DCT4_ReplyTestsStartup,	"\x35",0x02,0x01,ID_User3	},
 	{DCT4_ReplyTestsStartup,	"\x35",0x02,0x02,ID_User3	},
